#!/usr/bin/env python3

import os

os.makedirs("naughty", 0o700, True)
with open('naughty/file-with-escape-sequences-\x1b[31mred\x1b[m-color.txt', 'w+') as fi:
    fi.write('boo')

with open('naughty/file-with-escape-sequences-\x1b[31mred\x1b[m-color.log', 'w+') as fi:
    fi.write('2012-07-02 10:22:40,672:DEBUG:foo bar baz\n')

try:
    with open(b'naughty/text-file-with-invalid-utf-\xc3\x28', 'w+') as fi:
        fi.write('boo')

    with open(b'naughty/log-file-with-invalid-utf-\xc3\x28', 'w+') as fi:
        fi.write('2015-04-24T21:09:39.296 25376]ERROR:somemodule:Something very INFOrmative.')
except OSError as e:
    pass

with open('naughty/file-with-hidden-text.txt', 'w+') as fi:
    fi.write('Hello, \x1b[30;40mWorld!\x1b[m!\n')
    fi.write('Goodbye, \x1b[37;47mWorld!\x1b[m!\n')
    fi.write('That is not\b\b\ball\n')

with open('naughty/file-with-terminal-controls.txt', 'w+') as fi:
    fi.write('time for a reset \x1bckapow\n')
    fi.write('ding dong! \x07\n')




import glob
import csv
import sys
import json

def main(args):
    with open(args[1], 'w') as ofp:
        out = csv.writer(ofp)
        for format_path in sorted(glob.glob("%s/*.json" % args[2])):
            with open(format_path) as fp:
                format_dict = json.load(fp)

                for key in sorted(format_dict):
                    value = format_dict[key]
                    if not isinstance(value, dict):
                        continue
                    if 'title' not in value:
                        raise Exception("format '%s' is missing 'title'" % key)
                    out.writerow((value['title'], key, value['description']))

if __name__ == "__main__":
    sys.exit(main(sys.argv))



# -*- coding: utf-8 -*-
#
# lnav documentation build configuration file, created by
# sphinx-quickstart on Fri Jul 12 21:09:39 2013.
#
# This file is execfile()d with the current directory set to its containing dir.
#
# Note that not all possible configuration values are present in this
# autogenerated file.
#
# All configuration values have a default; values that are commented out
# serve to show the default.

import sys, os

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
this_dir = os.path.abspath('.')
src_dir = os.path.join(this_dir, "..", "..", "src")
sys.path.insert(0, this_dir)
sys.path.insert(0, src_dir)

import format2csv

format2csv.main(["",
                 os.path.join(this_dir, "format-table.csv"),
                 os.path.join(src_dir, "formats")])

import re
from pygments.lexer import RegexLexer, words
from pygments.token import Punctuation, Whitespace, Text, Comment, Operator, \
    Keyword, Name, String, Literal, Number, Generic
from sphinx.highlighting import lexers


class CustSqliteLexer(RegexLexer):
    name = 'custsqlite'

    flags = re.IGNORECASE
    tokens = {
        'root': [
            (r'\s+', Text),
            (r'--.*\n?', Comment.Single),
            (r'#.*\n?', Comment.Single),
            (r'/\*', Comment.Multiline, 'multiline-comments'),
            (words((
                'ABORT',
                'ACTION',
                'ADD',
                'AFTER',
                'ALL',
                'ALTER',
                'ALWAYS',
                'ANALYZE',
                'AND',
                'AS',
                'ASC',
                'ATTACH',
                'AUTOINCREMENT',
                'BEFORE',
                'BEGIN',
                'BETWEEN',
                'BY',
                'CASCADE',
                'CASE',
                'CAST',
                'CHECK',
                'COLLATE',
                'COLUMN',
                'COMMIT',
                'CONFLICT',
                'CONSTRAINT',
                'CREATE',
                'CROSS',
                'CURRENT',
                'CURRENT_DATE',
                'CURRENT_TIME',
                'CURRENT_TIMESTAMP',
                'DATABASE',
                'DEFAULT',
                'DEFERRABLE',
                'DEFERRED',
                'DELETE',
                'DESC',
                'DETACH',
                'DISTINCT',
                'DO',
                'DROP',
                'EACH',
                'ELSE',
                'END',
                'ESCAPE',
                'EXCEPT',
                'EXCLUDE',
                'EXCLUSIVE',
                'EXISTS',
                'EXPLAIN',
                'FAIL',
                'FILTER',
                'FIRST',
                'FOLLOWING',
                'FOR',
                'FOREIGN',
                'FROM',
                'FULL',
                'GENERATED',
                'GLOB',
                'GROUP',
                'GROUPS',
                'HAVING',
                'IF',
                'IGNORE',
                'IMMEDIATE',
                'IN',
                'INDEX',
                'INDEXED',
                'INITIALLY',
                'INNER',
                'INSERT',
                'INSTEAD',
                'INTERSECT',
                'INTO',
                'IS',
                'ISNULL',
                'JOIN',
                'KEY',
                'LAST',
                'LEFT',
                'LIKE',
                'LIMIT',
                'MATCH',
                'NATURAL',
                'NO',
                'NOT',
                'NOTHING',
                'NOTNULL',
                'NULL',
                'NULLS',
                'OF',
                'OFFSET',
                'ON',
                'OR',
                'ORDER',
                'OTHERS',
                'OUTER',
                'OVER',
                'PARTITION',
                'PLAN',
                'PRAGMA',
                'PRECEDING',
                'PRIMARY',
                'QUERY',
                'RAISE',
                'RANGE',
                'RECURSIVE',
                'REFERENCES',
                'REGEXP',
                'REINDEX',
                'RELEASE',
                'RENAME',
                'REPLACE',
                'RESTRICT',
                'RIGHT',
                'ROLLBACK',
                'ROW',
                'ROWS',
                'SAVEPOINT',
                'SELECT',
                'SET',
                'TABLE',
                'TEMP',
                'TEMPORARY',
                'THEN',
                'TIES',
                'TO',
                'TRANSACTION',
                'TRIGGER',
                'UNBOUNDED',
                'UNION',
                'UNIQUE',
                'UPDATE',
                'USING',
                'VACUUM',
                'VALUES',
                'VIEW',
                'VIRTUAL',
                'WHEN',
                'WHERE',
                'WINDOW',
                'WITH',
                'WITHOUT'), suffix=r'\b'),
             Keyword),
            (words((
                'ARRAY', 'BIGINT', 'BINARY', 'BIT', 'BLOB', 'BOOLEAN', 'CHAR',
                'CHARACTER', 'DATE', 'DEC', 'DECIMAL', 'FLOAT', 'INT', 'INTEGER',
                'INTERVAL', 'NUMBER', 'NUMERIC', 'REAL', 'SERIAL', 'SMALLINT',
                'VARCHAR', 'VARYING', 'INT8', 'SERIAL8', 'TEXT'), suffix=r'\b'),
             Name.Builtin),
            (r'[+*/<>=~!@#%^&|`?-]', Operator),
            (r'[0-9]+', Number.Integer),
            # TODO: Backslash escapes?
            (r"'(''|[^'])*'", String.Single),
            (r'"(""|[^"])*"', String.Symbol),  # not a real string literal in ANSI SQL
            (r'[a-z_][\w$]*', Name),  # allow $s in strings for Oracle
            (r'\$[a-z_]+', Name),
            (r'[;:()\[\],.]', Punctuation)
        ],
        'multiline-comments': [
            (r'/\*', Comment.Multiline, 'multiline-comments'),
            (r'\*/', Comment.Multiline, '#pop'),
            (r'[^/*]+', Comment.Multiline),
            (r'[/*]', Comment.Multiline)
        ]
    }

    def analyse_text(text):
        return 0.01


lexers['custsqlite'] = CustSqliteLexer(startinline=True)

# -- General configuration -----------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
# needs_sphinx = '1.0'

# Add any Sphinx extension module names here, as strings. They can be extensions
# coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = [
    'sphinx.ext.coverage',
    'sphinx_copybutton',
    'sphinx-jsonschema',
    'sphinx-prompt',
    '_ext.lnavlexer',
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix of source filenames.
source_suffix = '.rst'

# The encoding of source files.
# source_encoding = 'utf-8-sig'

# The master toctree document.
master_doc = 'index'

# General information about the project.
project = u'lnav'
copyright = u'2023, Tim Stack'

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#
# The short X.Y version.
version = '0.12'
# The full version, including alpha/beta/rc tags.
release = '0.12.3'

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
# language = None

# There are two options for replacing |today|: either, you set today to some
# non-false value, then it is used:
# today = ''
# Else, today_fmt is used as the format for a strftime call.
# today_fmt = '%B %d, %Y'

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
exclude_patterns = []

# The reST default role (used for this markup: `text`) to use for all documents.
# default_role = None

# If true, '()' will be appended to :func: etc. cross-reference text.
# add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
# add_module_names = True

# If true, sectionauthor and moduleauthor directives will be shown in the
# output. They are ignored by default.
# show_authors = False

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# A list of ignored prefixes for module index sorting.
# modindex_common_prefix = []

# -- Options for HTML output ---------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
html_theme = 'furo'

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
html_theme_options = {
}

# Add any paths that contain custom themes here, relative to this directory.
# html_theme_path = []

# The name for this set of Sphinx documents.  If None, it defaults to
# "<project> v<release> documentation".
html_title = f"lnav v{release}"

# A shorter title for the navigation bar.  Default is the same as html_title.
# html_short_title = None

# The name of an image file (relative to this directory) to place at the top
# of the sidebar.
# html_logo = None

# The name of an image file (within the static path) to use as favicon of the
# docs.  This file should be a Windows icon file (.ico) being 16x16 or 32x32
# pixels large.
# html_favicon = None

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']


def setup(app):
    app.add_css_file('theme_overrides.css')


# html_context = {
#    'css_files': [
#        '_static/theme_overrides.css',  # override wide tables in RTD theme
#    ],
# }

# If not '', a 'Last updated on:' timestamp is inserted at every page bottom,
# using the given strftime format.
# html_last_updated_fmt = '%b %d, %Y'

# If true, SmartyPants will be used to convert quotes and dashes to
# typographically correct entities.
# html_use_smartypants = True

# Custom sidebar templates, maps document names to template names.
# html_sidebars = {}

# Additional templates that should be rendered to pages, maps page names to
# template names.
# html_additional_pages = {}

# If false, no module index is generated.
# html_domain_indices = True

# If false, no index is generated.
# html_use_index = True

# If true, the index is split into individual pages for each letter.
# html_split_index = False

# If true, links to the reST sources are added to the pages.
# html_show_sourcelink = True

# If true, "Created using Sphinx" is shown in the HTML footer. Default is True.
# html_show_sphinx = True

# If true, "(C) Copyright ..." is shown in the HTML footer. Default is True.
# html_show_copyright = True

# If true, an OpenSearch description file will be output, and all pages will
# contain a <link> tag referring to it.  The value of this option must be the
# base URL from which the finished HTML is served.
# html_use_opensearch = ''

# This is the file name suffix for HTML files (e.g. ".xhtml").
# html_file_suffix = None

# Output file base name for HTML help builder.
htmlhelp_basename = 'lnavdoc'

# -- Options for LaTeX output --------------------------------------------------

latex_elements = {
    # The paper size ('letterpaper' or 'a4paper').
    # 'papersize': 'letterpaper',

    # The font size ('10pt', '11pt' or '12pt').
    # 'pointsize': '10pt',

    # Additional stuff for the LaTeX preamble.
    # 'preamble': '',
}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title, author, documentclass [howto/manual]).
latex_documents = [
    ('index', 'lnav.tex', u'lnav Documentation',
     u'Tim Stack', 'manual'),
]

# The name of an image file (relative to this directory) to place at the top of
# the title page.
# latex_logo = None

# For "manual" documents, if this is true, then toplevel headings are parts,
# not chapters.
# latex_use_parts = False

# If true, show page references after internal links.
# latex_show_pagerefs = False

# If true, show URL addresses after external links.
# latex_show_urls = False

# Documents to append as an appendix to all manuals.
# latex_appendices = []

# If false, no module index is generated.
# latex_domain_indices = True


# -- Options for manual page output --------------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    ('index', 'lnav', u'lnav Documentation',
     [u'Tim Stack'], 1)
]

# If true, show URL addresses after external links.
# man_show_urls = False


# -- Options for Texinfo output ------------------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
    ('index', 'lnav', u'lnav Documentation',
     u'Tim Stack', 'lnav', 'One line description of project.',
     'Miscellaneous'),
]

# Documents to append as an appendix to all manuals.
# texinfo_appendices = []

# If false, no module index is generated.
# texinfo_domain_indices = True

# How to display URL addresses: 'footnote', 'no', or 'inline'.
# texinfo_show_urls = 'footnote'



from werkzeug.middleware.proxy_fix import ProxyFix
from flask import Flask, request, send_file, after_this_request
import os
import glob
import uuid
from datetime import datetime
import zipfile
import spookyhash

app = Flask(__name__)
app.wsgi_app = ProxyFix(
    app.wsgi_app, x_for=2, x_proto=2, x_host=2, x_prefix=2
)

ROOT_PAGE = """
<html>
<head>
<title>lnav crash upload site</title>
</head>

<body>
You can help improve <b>lnav</b> by uploading your crash logs by running:
<pre>
lnav -m crash upload
</pre>
</body>
</html>
"""


# Function to check free space on filesystem
def check_free_space():
    statvfs = os.statvfs('/logs')
    # Calculate free space in MB
    free_space_mb = (statvfs.f_bsize * statvfs.f_bavail) / (1024 * 1024)
    return free_space_mb


@app.route('/crash', methods=['POST'])
def crash_handler():
    # Check free space on filesystem
    free_space_mb = check_free_space()
    if free_space_mb < 100:
        return 'Insufficient free space on the filesystem!\n', 500

    # Retrieve the secret value from the environment variable
    lnav_secret_env = os.environ.get('LNAV_SECRET')

    # Check if header 'lnav-secret' exists and has the correct value
    if request.headers.get('lnav-secret') == lnav_secret_env:
        # Check if content length is provided
        if 'Content-Length' not in request.headers:
            return 'Content length header is missing!', 400

        # Get the content length
        content_length = int(request.headers['Content-Length'])

        # Check if content length is zero
        if content_length == 0:
            return 'Empty request body!', 400

        # Check if content length exceeds 10MB
        if content_length > 10 * 1024 * 1024:  # 10MB limit
            return 'Content size exceeds the limit of 10MB!', 413

        # Get the content from the request body
        content = request.data

        nonce = request.headers.get('X-lnav-nonce', '')
        crash_hash = request.headers.get('X-lnav-hash', '')
        if not crash_hash.startswith("0000"):
            return "Invalid proof of work hash", 401

        sh = spookyhash.Hash128()
        sh.update(nonce.encode('utf-8'))
        sh.update(content)
        verify_hash = sh.hexdigest()
        if verify_hash != crash_hash:
            return "Proof of work hash does not match", 401

        # Generate a unique ID
        unique_id = str(uuid.uuid4())

        # Get the current time
        current_time = datetime.now().strftime('%Y-%m-%d_%H-%M-%S')

        # Construct the file name with current time and unique ID
        file_name = f'crash_log_{current_time}_{unique_id}.txt'
        full_path = os.path.join("/logs", file_name)

        # Save the content to the file in the current directory
        with open(full_path, 'wb') as f:
            f.write(content)

        return 'Data saved successfully!', 200
    else:
        return 'Unauthorized access!', 401


@app.route('/download_crashes', methods=['GET'])
def download_crashes():
    # Retrieve the secret value for downloading from the environment variable
    lnav_download_secret_env = os.environ.get('LNAV_DOWNLOAD_SECRET')

    # Check if header 'lnav-secret' exists and has the correct value for downloading
    if request.headers.get('lnav-secret') == lnav_download_secret_env:
        # Get all the files in the current directory
        crash_files = glob.glob("/logs/crash_log_*")

        # Generate a unique ID for the zip file
        zip_id = str(uuid.uuid4())

        # Construct the zip file name
        zip_file_name = f'crash_archive_{zip_id}.zip'

        # Create a new zip file
        with zipfile.ZipFile(zip_file_name, 'w') as zipf:
            # Add each crash file to the zip file
            for crash_file in crash_files:
                zipf.write(crash_file)

        # Delete the crash files
        for crash_file in crash_files:
            os.remove(crash_file)

        @after_this_request
        def remove_zip(response):
            try:
                os.remove(zip_file_name)
            except Exception as e:
                app.logger.error(f"Error removing zip file: {e}")
            return response

        # Send the zip file as a response
        return send_file(zip_file_name, as_attachment=True, download_name=zip_file_name)

    else:
        return 'Unauthorized access!\n', 401


@app.route('/', methods=['GET'])
def root_page():
    return ROOT_PAGE, 200


if __name__ == '__main__':
    # Run the Flask app
    app.run()



import datetime
import os
import random
import shutil
import sys

MSGS = [
    "Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua.",
    "Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat.",
    "Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur.",
    "Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum.",
]

GLOG_DATE_FMT = "%Y%m%d %H:%M:%S"

START_TIME = datetime.datetime.fromtimestamp(1490191111)

try:
    shutil.rmtree("/tmp/demo")
    os.makedirs("/tmp/demo")
except OSError:
    pass

PIDS = [
    "123",
    "123",
    "123",
    "121",
    "124",
    "123",
    "61456",
    "61456",
    "61457",
]

LOG_LOCS = [
    "demo.cc:123",
    "demo.cc:352",
    "loader.cc:13",
    "loader.cc:552",
    "blaster.cc:352",
    "blaster.cc:112",
    "blaster.cc:6782",
]

CURR_TIME = START_TIME
for _index in range(0, int(sys.argv[1])):
    CURR_TIME += datetime.timedelta(seconds=random.randrange(1, 22))
    print("I%s.%06d %s %s] %s" % (
        CURR_TIME.strftime(GLOG_DATE_FMT),
        random.randrange(0, 100000),
        random.choice(PIDS),
        random.choice(LOG_LOCS),
        random.choice(MSGS)))



#!/usr/bin/env python3

import sys
import html
import time
import json
import urllib.request

sys.stderr.write("reading stdin\n")
inj = json.load(sys.stdin)
sys.stderr.write("reading stdin done\n")

if False:
    print("Hello, World!")
    sys.exit()

RESOLVING_SERVICE_URL = "http://btresolver.eng.vmware.com:80/"
DEFAULT_TIMEOUT = 10

req_url = "%s%s" % (RESOLVING_SERVICE_URL, "async_resolve_text_bts")
log_msg = inj['log_msg']
index = log_msg.find('[context]')
if index != -1:
    log_msg = log_msg[index:]
body = json.dumps([log_msg])
req = urllib.request.Request(req_url, body.encode('utf-8'))
resp = urllib.request.urlopen(req, timeout=DEFAULT_TIMEOUT)
resolve_content = json.loads(resp.read())

sys.stderr.write("resolve %s\n" % resolve_content)
if not resolve_content['success']:
    print(resolve_content['errorString'])
    sys.exit(1)

time.sleep(0.5)

delay = 1

done = False
while not done:
    get_url = "%s%s" % (RESOLVING_SERVICE_URL, "get_task")
    body = resolve_content['returnValue']['TaskIds'][0]
    get_req = urllib.request.Request(get_url, body.encode('utf-8'))
    get_resp = urllib.request.urlopen(get_req, timeout=DEFAULT_TIMEOUT)
    get_content = json.loads(get_resp.read())
    sys.stderr.write("get %s\n" % get_content)
    if get_content['returnValue']['state'] == 'RUNNING':
        if get_content['returnValue']['exception'] is None:
            time.sleep(delay)
            if delay < 10:
                delay = delay * 2
        else:
            print("<pre>\n%s</pre>" % html.escape(get_content['returnValue']['exception']))
            done = True
    elif get_content['returnValue']['state'] == 'COMPLETED':
        if get_content['returnValue'].get('output') is not None:
            print("<pre>\n%s</pre>" % html.escape(get_content['returnValue']['output']))
        elif get_content['returnValue'].get('exception') is not None:
            print("<pre>\n%s</pre>" % html.escape(get_content['returnValue']['exception']))
        done = True
